@@ -1,16 +1,31 @@
 //! Quadratic functions.
+use std::fmt::Display;
+
 use crate::math::general::CommonPowers;
 /// A struct for storing quadratic equations of the form `f(x) = ax² + bx + c`.
 #[derive(Clone, Copy, Debug, PartialEq)]
-pub struct QuadraticEquation {
-    a:         f64,
-    b:         f64,
-    c:         f64,
-    vertex:    Option<(f64, f64)>,
-    solutions: Option<(Option<f64>, Option<f64>)>
+pub struct QuadraticEquation<T> {
+    a:         T,
+    b:         T,
+    c:         T,
+    vertex:    Option<(T, T)>,
+    solutions: Option<(Option<T>, Option<T>)>
 }
 
-impl QuadraticEquation {
+impl<T: Copy +
+        Clone +
+        From<u8> +
+        From<f64> +
+        PartialEq +
+        PartialOrd +
+        CommonPowers +
+        std::ops::Mul<Output = T> +
+        std::ops::Add<Output = T> +
+        std::ops::Sub<Output = T> +
+        std::ops::Div<Output = T> +
+        std::ops::Neg<Output = T>> QuadraticEquation<T>
+        where
+        f64: From<T> {
     /// Create a new `QuadraticEquation` with the values `a = 1, b = 0, c = 0`.
     /// 
     /// # Examples
@@ -32,12 +47,12 @@ impl QuadraticEquation {
     /// ```
     #[inline]
     #[must_use]
-    pub const fn new() -> QuadraticEquation {
-        QuadraticEquation { a:         1.0,
-                            b:         0.0,
-                            c:         0.0,
-                            vertex:    Some((0.0, 0.0)),
-                            solutions: Some((Some(0.0), None)) }
+    pub fn new() -> QuadraticEquation<T> {
+        QuadraticEquation { a:         T::from(1),
+                            b:         T::from(0),
+                            c:         T::from(0),
+                            vertex:    Some((T::from(0), T::from(0))),
+                            solutions: Some((Some(T::from(0)), None)) }
     }
     /// Create a new `QuadraticEquation` from coefficients.
     /// 
@@ -53,8 +68,8 @@ impl QuadraticEquation {
     /// ```
     #[inline]
     #[must_use]
-    pub fn new_from_coefficients(a: f64, b: f64, c: f64) -> QuadraticEquation {
-        if a == 0.0
+    pub fn new_from_coefficients(a: T, b: T, c: T) -> QuadraticEquation<T> {
+        if a == T::from(0)
         { panic!("a was zero and is thus not allowed."); }
         QuadraticEquation { a,
                             b,
@@ -75,7 +90,7 @@ impl QuadraticEquation {
     /// ```
     #[inline]
     #[must_use]
-    pub const fn a(&self) -> f64 {
+    pub fn a(&self) -> T {
         self.a
     }
     /// Get `b` of a `QuadraticEquation`.
@@ -91,7 +106,7 @@ impl QuadraticEquation {
     /// ```
     #[inline]
     #[must_use]
-    pub const fn b(&self) -> f64 {
+    pub fn b(&self) -> T {
         self.b
     }
     /// Get `c` of a `QuadraticEquation`.
@@ -107,7 +122,7 @@ impl QuadraticEquation {
     /// ```
     #[inline]
     #[must_use]
-    pub const fn c(&self) -> f64 {
+    pub fn c(&self) -> T {
         self.c 
     }
     /// Set `c` of a `QuadraticEquation`.
@@ -126,8 +141,8 @@ impl QuadraticEquation {
     /// assert_eq!(-1.0, f_x.a());
     /// ```
     #[inline]
-    pub fn set_a(&mut self, value: f64) {
-        if value == 0.0
+    pub fn set_a(&mut self, value: T) {
+        if value == T::from(0)
         { panic!("a was zero and is thus not allowed."); }
         self.a = value;
     }
@@ -145,7 +160,7 @@ impl QuadraticEquation {
     /// assert_eq!(-1.0, f_x.b());
     /// ```
     #[inline] 
-    pub fn set_b(&mut self, value: f64) {
+    pub fn set_b(&mut self, value: T) {
         self.b = value;
     }
     /// Set `c` of a `QuadraticEquation`.
@@ -162,7 +177,7 @@ impl QuadraticEquation {
     /// assert_eq!(-1.0, f_x.c());
     /// ```
     #[inline]
-    pub fn set_c(&mut self, value: f64) {
+    pub fn set_c(&mut self, value: T) {
         self.c = value;
     }
     /// Get the solutions of a quadratic equation.
@@ -176,14 +191,14 @@ impl QuadraticEquation {
     /// 
     /// assert_eq!(Some((Some(3.0), Some(-1.0))), f_x.get_solutions());
     /// ```
-    pub fn get_solutions(&mut self) -> Option<(Option<f64>, Option<f64>)> {
+    pub fn get_solutions(&mut self) -> Option<(Option<T>, Option<T>)> {
         if self.solutions.is_some()
         { return self.solutions; }
-        let discriminant = self.b.square() - 4.0 * self.a * self.c;
-        if discriminant < 0.0
+        let discriminant = self.b.square() - T::from(4) * self.a * self.c;
+        if discriminant < T::from(0)
         { return None; }
-        let x_1 = (- self.b + discriminant.sqrt()) / (2.0 * self.a);
-        let x_2 = (- self.b - discriminant.sqrt()) / (2.0 * self.a);
+        let x_1 = (- self.b + T::from(f64::from(discriminant).sqrt())) / (T::from(2) * self.a);
+        let x_2 = (- self.b - T::from(f64::from(discriminant).sqrt())) / (T::from(2) * self.a);
         
         match x_1 == x_2 {
             true  => { self.solutions = Some((Some(x_1), None)); }
@@ -204,10 +219,10 @@ impl QuadraticEquation {
     /// ```
     #[inline]
     #[must_use]
-    pub fn get_vertex(&mut self) -> (f64, f64) {
+    pub fn get_vertex(&mut self) -> (T, T) {
         if self.vertex.is_some()
         { return unsafe { self.vertex.unwrap_unchecked() }; }
-        let x = - self. b / (2.0 * self.a);
+        let x = - self. b / (T::from(2) * self.a);
         self.vertex = Some((x, self.a * x.square() + self.b * x + self.c));
         unsafe { self.vertex.unwrap_unchecked() }
     }
@@ -224,22 +239,27 @@ impl QuadraticEquation {
     /// ```
     #[inline]
     #[must_use]
-    pub fn get_fun_val_of<T>(&self, x: T) -> f64
-    where f64: From<T> {
-        let _x = f64::from(x);
+    pub fn get_fun_val_of<S: CommonPowers>(&self, x: S) -> T
+    where f64: From<T>,
+          T: From<S> {
+        let _x = T::from(x);
         self.a * _x.square() + self.b * _x + self.c
     }
 }
 
-impl std::fmt::Display for QuadraticEquation {
+impl<T: Display +
+        std::ops::Neg<Output = T> +
+        PartialOrd +
+        From<u8> +
+        Copy> std::fmt::Display for QuadraticEquation<T> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         let res;
         let _  = write!(f, "f(x) = {}x^2", self.a);
-        if self.b < 0.0
+        if self.b < T::from(0)
         { let _ = write!(f, " - {}x", - self.b); }
         else
         { let _ = write!(f, " + {}x", self.b); }
-        if self.c < 0.0
+        if self.c < T::from(0)
         { res = write!(f, " - {}", - self.c); }
         else
         { res = write!(f, " + {}", self.c); }
